@@ -28,7 +28,10 @@
 #include "WPXStream.h"
 #include <stdio.h>
 #include <string>
+#include <algorithm>
 #include "WPXString.h"
+#include "WPXEncryption.h"
+#include "libwpd_types.h"
 
 /* Various functions/defines that need not/should not be exported externally */
 
@@ -46,7 +49,7 @@
 #define WPD_CHECK_FILE_READ_ERROR(v,num_elements) if (v != num_elements) {\
  WPD_DEBUG_MSG(("X_CheckFileReadElementError: %d\n", __LINE__)); throw FileException(); }
 
-#define DELETEP(m) if (m) { delete m; m = NULL; }
+#define DELETEP(m) if (m) { delete m; m = 0; }
  
 #ifdef DEBUG
 #define WPD_DEBUG_MSG(M) printf M
@@ -74,28 +77,32 @@
                   (((uint8_t const *)(p))[1] << 16) |    \
                   (((uint8_t const *)(p))[0] << 24))
 
+#define WPD_NUM_ELEMENTS(array) sizeof(array)/sizeof(array[0])
+
 // add more of these as needed for byteswapping
 // (the 8-bit functions are just there to make things consistent)
-int8_t read8(WPXInputStream *input); 
-uint8_t readU8(WPXInputStream *input); 
-uint16_t readU16(WPXInputStream *input, bool bigendian=false);
-uint32_t readU32(WPXInputStream *input, bool bigendian=false);
+uint8_t readU8(WPXInputStream *input, WPXEncryption *encryption); 
+uint16_t readU16(WPXInputStream *input, WPXEncryption *encryption, bool bigendian=false);
+uint32_t readU32(WPXInputStream *input, WPXEncryption *encryption, bool bigendian=false);
 
-WPXString readPascalString(WPXInputStream *input);
-WPXString readCString(WPXInputStream *input);
+WPXString readPascalString(WPXInputStream *input, WPXEncryption *encryption);
+WPXString readCString(WPXInputStream *input, WPXEncryption *encryption);
 
 void appendUCS4(WPXString &str, uint32_t ucs4);
 
 // Various helper structures for the libwpd parser..
 
-int extendedCharacterWP6ToUCS2(uint8_t character, uint8_t characterSet,
-			    const uint16_t **chars);
+int extendedCharacterWP6ToUCS4(uint8_t character, uint8_t characterSet,
+			    const uint32_t **chars);
 
-int extendedCharacterWP5ToUCS2(uint8_t character, uint8_t characterSet,
-			    const uint16_t **chars);
+int extendedCharacterWP5ToUCS4(uint8_t character, uint8_t characterSet,
+			    const uint32_t **chars);
+
+int appleWorldScriptToUCS4(uint16_t character, const uint32_t **chars);
 
 uint16_t fixedPointToWPUs(const uint32_t fixedPointNumber);
-float fixedPointToFloat(const uint32_t fixedPointNumber);
+double fixedPointToDouble(const uint32_t fixedPointNumber);
+double wpuToFontPointSize(const uint16_t wpuNumber);
 
 enum WPXFileType { WP6_DOCUMENT, WP5_DOCUMENT, WP42_DOCUMENT, OTHER };
 enum WPXNumberingType { ARABIC, LOWERCASE, UPPERCASE, LOWERCASE_ROMAN, UPPERCASE_ROMAN };
@@ -103,12 +110,21 @@ enum WPXNoteType { FOOTNOTE, ENDNOTE };
 enum WPXHeaderFooterType { HEADER, FOOTER };
 enum WPXHeaderFooterInternalType { HEADER_A, HEADER_B, FOOTER_A, FOOTER_B, DUMMY };
 enum WPXHeaderFooterOccurence { ODD, EVEN, ALL, NEVER };
+enum WPXPageNumberPosition { PAGENUMBER_POSITION_NONE = 0, PAGENUMBER_POSITION_TOP_LEFT, PAGENUMBER_POSITION_TOP_CENTER, 
+			     PAGENUMBER_POSITION_TOP_RIGHT, PAGENUMBER_POSITION_TOP_LEFT_AND_RIGHT, 
+			     PAGENUMBER_POSITION_BOTTOM_LEFT, PAGENUMBER_POSITION_BOTTOM_CENTER, 
+			     PAGENUMBER_POSITION_BOTTOM_RIGHT, PAGENUMBER_POSITION_BOTTOM_LEFT_AND_RIGHT, 
+			     PAGENUMBER_POSITION_TOP_INSIDE_LEFT_AND_RIGHT, 
+			     PAGENUMBER_POSITION_BOTTOM_INSIDE_LEFT_AND_RIGHT };
+
 enum WPXFormOrientation { PORTRAIT, LANDSCAPE };
 enum WPXTabAlignment { LEFT, RIGHT, CENTER, DECIMAL, BAR };
 enum WPXVerticalAlignment { TOP, MIDDLE, BOTTOM, FULL };
 
 enum WPXTextColumnType { NEWSPAPER, NEWSPAPER_VERTICAL_BALANCE, PARALLEL, PARALLEL_PROTECT };
 
+enum WPXSubDocumentType { WPX_SUBDOCUMENT_NONE, WPX_SUBDOCUMENT_HEADER_FOOTER, WPX_SUBDOCUMENT_NOTE, WPX_SUBDOCUMENT_TEXT_BOX, WPX_SUBDOCUMENT_COMMENT_ANNOTATION };
+
 // ATTRIBUTE bits
 #define WPX_EXTRA_LARGE_BIT 1
 #define WPX_VERY_LARGE_BIT 2
@@ -179,9 +195,9 @@ typedef struct _WPXColumnDefinition WPXColumnDefinition;
 struct _WPXColumnDefinition
 {
 	_WPXColumnDefinition(); // initializes all values to 0
-	float m_width;
-	float m_leftGutter;
-	float m_rightGutter;
+	double m_width;
+	double m_leftGutter;
+	double m_rightGutter;
 };
 
 typedef struct _WPXColumnProperties WPXColumnProperties;
@@ -195,9 +211,9 @@ struct _WPXColumnProperties
 typedef struct _WPXTabStop WPXTabStop;
 struct _WPXTabStop
 {
-	_WPXTabStop(float position, WPXTabAlignment alignment, uint16_t leaderCharacter, uint8_t leaderNumSpaces);
+	_WPXTabStop(double position, WPXTabAlignment alignment, uint16_t leaderCharacter, uint8_t leaderNumSpaces);
 	_WPXTabStop();
-	float m_position;
+	double m_position;
 	WPXTabAlignment m_alignment;
 	uint16_t m_leaderCharacter;
 	uint8_t m_leaderNumSpaces;
@@ -207,27 +223,30 @@ struct _WPXTabStop
 
 class VersionException
 {
-	// needless to say, we could flesh this class out a bit
 };
 
 class FileException
 {
-	// needless to say, we could flesh this class out a bit
 };
 
 class ParseException
 {
-	// needless to say, we could flesh this class out a bit
 };
 
 class GenericException
 {
-	// needless to say, we could flesh this class out a bit
 };
 
 class UnsupportedEncryptionException
 {
-	// needless to say, we could flesh this class out a bit
+};
+
+class SupportedEncryptionException
+{
+};
+
+class WrongPasswordException
+{
 };
 
 // Various usefull, but cheesey functions
@@ -236,6 +255,7 @@ int _extractNumericValueFromRoman(const char romanChar);
 int _extractDisplayReferenceNumberFromBuf(const WPXString &buf, const WPXNumberingType listType);
 WPXNumberingType _extractWPXNumberingTypeFromBuf(const WPXString &buf, const WPXNumberingType putativeWPXNumberingType);
 WPXString _numberingTypeToString(WPXNumberingType t);
-extern const uint16_t macintoshCharacterMap[];
+extern const uint32_t macRomanCharacterMap[];
+WPXString doubleToString(const double value);
 
 #endif /* LIBWPD_INTERNAL_H */
